#include "GuiGroup.h"

CGuiGroup::CGuiGroup(int x, int y, int width, int height) : CGuiParent(x, y, width, height)
{

}

CGuiGroup::~CGuiGroup()
{
	m_mapGuiList.FreePtr();
}

void CGuiGroup::Draw()
{
	for(it = m_mapGuiList.m_Map.begin(); it != m_mapGuiList.m_Map.end(); ++it)
	{
		CGuiParent* gui = it->second;
		gui->Draw();
	}
}

bool CGuiGroup::MouseButtonDown(MOUSE_BUTTON eMouseButton, int x, int y)
{
	for(it = m_mapGuiList.m_Map.begin(); it != m_mapGuiList.m_Map.end(); ++it)
	{
		CGuiParent* gui = it->second;
		if(gui->isValidArea(x, y))
		{
			if(gui->MouseButtonDown(eMouseButton, x, y))
			{
				SetVisible(false);
				return true;
			}
		}
	}
	return false;
}

bool CGuiGroup::MouseMove(int x, int y)
{
	for(it = m_mapGuiList.m_Map.begin(); it != m_mapGuiList.m_Map.end(); ++it)
	{
		CGuiParent* gui = it->second;
		if(gui->isValidArea(x, y))
		{
			if(gui->MouseMove(x, y))
			{
				SetMouseOver(gui);
				return true;
			}
		}
	}
	SetMouseOver((CGuiParent*)NULL);
	return false;
}

void CGuiGroup::SetMouseOver(CGuiParent* mouseovergui)
{
	for(it = m_mapGuiList.m_Map.begin(); it != m_mapGuiList.m_Map.end(); ++it)
	{
		CGuiParent* gui = it->second;
		if(gui == mouseovergui){
			gui->SetMouseOver(true);
		}
		else{
			gui->SetMouseOver(false);
		}
	}
}

void CGuiGroup::SetMouseOver(bool bMouseOver)
{
	if(bMouseOver == false)
	{
		SetMouseOver((CGuiParent*)NULL);
	}
}

bool CGuiGroup::AddGui(CGuiParent* gui)
{
	if(!m_mapGuiList.Exist(gui->GetIdx()))
	{
		return m_mapGuiList.Add(gui->GetIdx(), gui);
	}
	return false;
}

void CGuiGroup::SetX(int x)
{
	CGuiParent::SetX(x);
	for(it = m_mapGuiList.m_Map.begin(); it != m_mapGuiList.m_Map.end(); ++it)
	{
		CGuiParent* gui = it->second;
		gui->SetX(x);
	}
}

void CGuiGroup::SetY(int y)
{
	CGuiParent::SetY(y);
	for(it = m_mapGuiList.m_Map.begin(); it != m_mapGuiList.m_Map.end(); ++it)
	{
		CGuiParent* gui = it->second;
		gui->SetY(y);
		y += gui->GetHeight();
	}
}