#include "GuiManage.h"

CGuiManage::CGuiManage(int* piScrollX, int* piScrollY, bool bPreemitiveMode)
{
	for(int i = 0; i < 10000; i++)
		m_qIdx.push(i);

	m_piScrollX = piScrollX;
	m_piScrollY = piScrollY;
	m_bPreemitiveMode = bPreemitiveMode;
	m_pcPreemitiveGui = NULL;
	m_pcFocusGui = NULL;
}

CGuiManage::~CGuiManage()
{
	m_mapGuiParent.FreePtr();
}

void CGuiManage::AdjustPos(int &x, int &y)
{
	if(!m_bPreemitiveMode)
	{
		if(m_piScrollX)
			x +=* m_piScrollX;

		if(m_piScrollY)
			y +=* m_piScrollY;
	}
}

bool CGuiManage::AddGui(CGuiParent* guiparent)
{
	if(!m_mapGuiParent.Exist(guiparent->GetIdx()))
	{
		m_vGuiParent.push_back(guiparent);
		return m_mapGuiParent.Add(guiparent->GetIdx(), guiparent);
	}
	delete guiparent;
	return false;
}

bool CGuiManage::RemoveGui(int idx)
{
	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent->GetIdx() == idx){
			m_qIdx.push(idx);
			m_vGuiParent.erase(it);
			return m_mapGuiParent.RemovePtr(idx);
		}
	}
	return false;
}

bool CGuiManage::PreemitiveDraw() // ׸ .  ׸ Gui   ִٸ, װ͸ ׸.
{
	if(m_pcPreemitiveGui)
	{
		m_pcPreemitiveGui->Draw();
		return true;
	}
	return false;
}

void CGuiManage::Draw() //ʰ  ֵ  ׷~~ 
{
	for(reverse_it = m_vGuiParent.rbegin(); reverse_it != m_vGuiParent.rend(); ++reverse_it)
	{
		CGuiParent* guiparent = (*reverse_it);
		if(guiparent->isVisible())
			guiparent->Draw();
	}
}

bool CGuiManage::MouseMove(int x, int y) //ʰ  ֵ  ׷~~ 
{
	AdjustPos(x,y);
	if(m_pcPreemitiveGui) return m_pcPreemitiveGui->MouseMove(x, y);
	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent->isValidArea(x, y)){
			if(guiparent->MouseMove(x, y))
			{
				SetMouseOver(guiparent); 
				return true;
			}
		}
	}
	SetMouseOver(NULL);
	return false;
}

void CGuiManage::SetMouseOver(CGuiParent* gui)
{
	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent == gui){
			guiparent->SetMouseOver(true);
		}
		else{
			guiparent->SetMouseOver(false);
		}
	}
}

bool CGuiManage::KeyDown(int keyvalue)
{
	if(m_pcPreemitiveGui) return m_pcPreemitiveGui->KeyDown(keyvalue);

	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent->KeyDown(keyvalue))
			return true;
	}
	return false;
}

bool CGuiManage::KeyUp(int keyvalue)
{
	if(m_pcPreemitiveGui) return m_pcPreemitiveGui->KeyUp(keyvalue);

	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent->KeyUp(keyvalue))
			return true;
	}
	return false;
}

bool CGuiManage::MouseButtonUp(MOUSE_BUTTON eMouseButton, int x, int y)
{
	AdjustPos(x,y);

	if(m_pcPreemitiveGui) return m_pcPreemitiveGui->MouseButtonUp(eMouseButton, x, y);

	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent->isValidArea(x, y)){
			if(guiparent->MouseButtonUp(eMouseButton, x, y)){
				return true;
			}
		}
	}
	return false;
}

bool CGuiManage::MouseButtonDown(MOUSE_BUTTON eMouseButton, int x, int y)
{
	AdjustPos(x,y);

	if(m_pcPreemitiveGui) return m_pcPreemitiveGui->MouseButtonDown(eMouseButton, x, y);

	for(it = m_vGuiParent.begin(); it != m_vGuiParent.end(); ++it)
	{
		CGuiParent* guiparent = (*it);
		if(guiparent->isValidArea(x, y)){
			if(guiparent->MouseButtonDown(eMouseButton, x, y)){
				return true;
			}
		}
	}
	return false;
}



CGuiParent*  CGuiManage::GetGui(int idx)
{
	CGuiParent* guiparent;
	if(m_mapGuiParent.Get(idx, &guiparent))
	{
		return guiparent;
	}
	return NULL;
}