#include "SDLMain.h"

CSDLMain::CSDLMain(DWORD flag, Uint32 fps, CGuiManage* guimanage, int width, int height, BYTE bpp, DWORD videoflag) : CSDLVideo(width, height, bpp, videoflag)
{
	m_nMouseX = m_nMouseY = 0;
	m_nFps = fps;
	m_bInputMode = true;
	m_pcGuiManage = guimanage;
	if(m_pcGuiManage == NULL){
		fprintf(stderr, "CSDLMain::CSDLMain() m_pcGuiManage == NULL\n");
		exit(1);
	}

	m_Font = new CSDLFont;
	if(SDL_Init(flag) < 0) {
		fprintf(stderr, "CSDLMain::CSDLMain() Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}

	memset(m_KeyBuffer, 0, sizeof(m_KeyBuffer));
	memset(m_MouseButtonDown, 0, sizeof(m_MouseButtonDown));
}

CSDLMain::~CSDLMain()
{
	SAFE_DELETE(m_pcGuiManage);
	SAFE_DELETE(m_Font);
	atexit(SDL_Quit);
}

void CSDLMain::SetCaption(char* caption, char* iconfilename)
{
	SDL_WM_SetCaption(caption, iconfilename); 
}

void CSDLMain::Loop()
{
	while(1)
	{
		SDL_Event event;
		if(SDL_PollEvent(&event))
		{
			if(m_bInputMode)
			{
				switch (event.type)
				{
				case SDL_MOUSEMOTION:
					{
						if(!m_pcGuiManage->MouseMove(event.motion.x, event.motion.y))
							MouseMove(event.motion.x, event.motion.y);
					}
					break;
				case SDL_MOUSEBUTTONDOWN:
					{
						MOUSE_BUTTON eButton = (MOUSE_BUTTON)(event.button.button - 1);
						if(!m_pcGuiManage->MouseButtonDown(eButton, event.button.x, event.button.y))
							MouseButtonDown(eButton, event.button.x, event.button.y);
					}
					break;
				case SDL_MOUSEBUTTONUP:
					{
						MOUSE_BUTTON eButton = (MOUSE_BUTTON)(event.button.button - 1);
						if(!m_pcGuiManage->MouseButtonUp(eButton, event.button.x, event.button.y))
							MouseButtonUp(eButton, event.button.x, event.button.y);
					}
					break;
				case SDL_KEYDOWN:
					if(!m_pcGuiManage->KeyDown(event.key.keysym.sym))
						KeyDown(event.key.keysym.sym);
					break;
				case SDL_KEYUP:
					if(!m_pcGuiManage->KeyUp(event.key.keysym.sym))
						KeyUp(event.key.keysym.sym);
					break;
				}
			}

			switch(event.type)
			{
			case SDL_QUIT:
				exit(1);
				break;
			}
		}
		else
		{
			static Uint32 lasttime = SDL_GetTicks();
			Uint32 curtime = SDL_GetTicks();
			Uint32 gaptime = curtime-lasttime;
			if (gaptime >= 1000/m_nFps) 
			{
				lasttime += 1000/m_nFps;
				Act();
				if(m_pcGuiManage->PreemitiveDraw())
				{
					Flip();
				}
				else
				{
					Draw();
				}
			}
			else
			{
				SDL_Delay(max(1000/m_nFps - gaptime, 1));
			}
		}		
	}
}

void CSDLMain::EffectEnd()
{
	m_bInputMode = true;
	__super::EffectEnd();
}

void CSDLMain::ShowCursor(bool bShow)
{
	SDL_ShowCursor(bShow);
}

void CSDLMain::KeyDown(int keyvalue)
{
	if(0 <= keyvalue && keyvalue <= SDLK_LAST)
	{
		m_KeyBuffer[keyvalue] = true;
	}
}

void CSDLMain::KeyUp(int keyvalue)
{
	if(0 <= keyvalue && keyvalue <= SDLK_LAST)
	{
		m_KeyBuffer[keyvalue] = false;
	}
}

bool CSDLMain::isKeyDown(int keyvalue)
{
	if(0 <= keyvalue && keyvalue <= SDLK_LAST)
	{
		return m_KeyBuffer[keyvalue];
	}
	return false;
}

void CSDLMain::MouseMove(int x, int y)
{
	m_nMouseX = x;
	m_nMouseY = y;
}

void CSDLMain::MouseButtonDown(MOUSE_BUTTON eMouseButton, int x, int y)
{
	m_MouseButtonDown[eMouseButton] = true;
}

void CSDLMain::MouseButtonUp(MOUSE_BUTTON eMouseButton, int x, int y)
{
	m_MouseButtonDown[eMouseButton] = false;
}