#include "GameMain.h"
#include "Character.h"
#include "GuiPopup.h"
#include "Button.h"
#include "Tile.h"
#include "CharacterManage.h"
#include "TileManage.h"
#include "GuiImage.h"
#include "Npc.h"

const int TILE_SIZE = 64;

CGameMain::CGameMain(DWORD flag, unsigned long fps) : CSDLMain(flag, fps, new CGuiManage(&m_nScrollX, &m_nScrollY))
{
	m_pcSelectCharacter = NULL;
	m_pcEffectCharacter = NULL;
	m_nScrollX = m_nScrollY = 0;
	m_Font->OpenFont("YW_DOTUM.TTF", 15);
	SetCaption("Super Park Ji Sung War");

	new CTileManage((MAX_SCROLL_X / TILE_SIZE) + 1, (MAX_SCROLL_Y / TILE_SIZE) + 1);

	SDL_Surface* surface = LoadImg("zandi.png");
	GetTileManage()->AddTile(1, surface);
	surface = LoadImgAlpha("red.png");
	m_pcTileRed = GetTileManage()->AddTile(2, surface);

	surface = LoadImgColorKey("cursor.png");
	m_pcTileCursor = GetTileManage()->AddTile(3, surface);

	surface = LoadImgColorKey("select.png");
	m_pcTileSelect = GetTileManage()->AddTile(4, surface);

	surface = LoadImgAlpha("white.png");
	m_pcTileWhite = GetTileManage()->AddTile(5, surface);

	m_BackSurface = CreateSurface(SDL_HWSURFACE | SDL_DOUBLEBUF, MAX_SCROLL_X, MAX_SCROLL_Y);

	new CCharacterManage;
	surface = LoadImg(".png");
	CCharacter* character1 = new CCharacter(1, surface, 2, 2, 3);
	GetCharacterManage()->AddCharacterParent(character1);
	CCharacter* character2 = new CCharacter(2, surface, 3, 3, 5, 3);
	GetCharacterManage()->AddCharacterParent(character2);

	CGuiPopup* guipopup1 = new CGuiPopup(2*  TILE_SIZE, 2*  TILE_SIZE);
	guipopup1->SetVisible(FALSE);

	CGuiPopup* guipopup2 = new CGuiPopup(2*  TILE_SIZE, 2*  TILE_SIZE);
	guipopup2->SetVisible(FALSE);

	SDL_Surface* menu_surface1 = LoadImg("menu_move.png");
	SDL_Surface* menu_surface1_1 = LoadImg("menu_move_over.png");

	SDL_Surface* menu_surface2 = LoadImg("menu_attack.png");
	SDL_Surface* menu_surface2_1 = LoadImg("menu_attack_over.png");

	SDL_Surface* menu_surface3 = LoadImg("menu_info.png");
	SDL_Surface* menu_surface3_1 = LoadImg("menu_info_over.png");

	CButton<CCharacter>* guibutton1 = new CButton<CCharacter>(2*  TILE_SIZE, 2*  TILE_SIZE, menu_surface1, menu_surface1_1, &CCharacter::Move, character1);
	guipopup1->AddGui(guibutton1);
	CButton<CCharacter>* guibutton2 = new CButton<CCharacter>(2*  TILE_SIZE, 2*  TILE_SIZE, menu_surface2, menu_surface2_1, &CCharacter::Attack, character1);
	guipopup1->AddGui(guibutton2);
	CButton<CCharacter>* guibutton3 = new CButton<CCharacter>(2*  TILE_SIZE, 2*  TILE_SIZE, menu_surface3, menu_surface3_1, &CCharacter::Info, character1);
	guipopup1->AddGui(guibutton3);
	character1->SetGui(guipopup1);

	m_pcGuiManage->AddGui(guipopup1);

	CButton<CCharacter>* guibutton1_1 = new CButton<CCharacter>(2*  TILE_SIZE, 2*  TILE_SIZE, menu_surface1, menu_surface1_1, &CCharacter::Move, character2);
	guipopup2->AddGui(guibutton1_1);
	CButton<CCharacter>* guibutton2_1 = new CButton<CCharacter>(2*  TILE_SIZE, 2*  TILE_SIZE, menu_surface2, menu_surface2_1, &CCharacter::Attack, character2);
	guipopup2->AddGui(guibutton2_1);
	CButton<CCharacter>* guibutton3_1 = new CButton<CCharacter>(2*  TILE_SIZE, 2*  TILE_SIZE, menu_surface3, menu_surface3_1, &CCharacter::Info, character2);
	guipopup2->AddGui(guibutton3_1);
	character2->SetGui(guipopup2);

	m_pcGuiManage->AddGui(guipopup2);

	surface = LoadImg("midnight.png");
	m_pcLogoGui = new CGuiImage(0, 0, surface);

	surface = LoadImg("ȣ.png");
	CNpc* npc = new CNpc(3, surface, 7, 5, 3, 3);
	GetCharacterManage()->AddCharacterParent(npc);
}

CGameMain::~CGameMain()
{

}

void CGameMain::Act()
{
	
}

void CGameMain::Draw()
{
	GetTileManage()->Draw();
	GetCharacterManage()->Draw();
	DrawCursor();

	if(m_pcSelectCharacter)
	{
		GetGameMain()->DrawSelectCursor(m_pcSelectCharacter->GetDestRect());

		if(m_pcSelectCharacter->isMove())
		{
			m_pcSelectCharacter->DrawMoveArea();			
		}

		if(m_pcSelectCharacter->isAttack())
		{
			m_pcSelectCharacter->DrawAttackArea();
		}
	}

	m_pcGuiManage->Draw();

	SDL_Rect rectsurface = {0 + m_nScrollX, 0 + m_nScrollY, 640 + m_nScrollX, 480 + m_nScrollY};
	Blit(m_BackSurface, &rectsurface, 0, 0);

	SDL_Surface* surface = m_Font->SetText("MousePos x : %d, y : %d", m_nMouseX, m_nMouseY);
	if(surface)
	{
		Blit(surface, 20, 30);
	}
	Flip();
}

void CGameMain::DrawCursor()
{
	if(isShowCursor())
	{
		SDL_Rect destrect = {(m_nScrollX + m_nMouseX) / TILE_SIZE*  TILE_SIZE,  (m_nScrollY + m_nMouseY) / TILE_SIZE*  TILE_SIZE, (m_nScrollX + m_nMouseX) / TILE_SIZE*  TILE_SIZE + TILE_SIZE,  (m_nScrollY + m_nMouseY) / TILE_SIZE*  TILE_SIZE + TILE_SIZE};
		DrawBackSurface(m_pcTileCursor->GetSurface(), m_pcTileCursor->GetRect(), &destrect);
	}
}

void CGameMain::DrawSelectCursor(SDL_Rect* dstrect)
{
	DrawBackSurface(m_pcTileSelect->GetSurface(), m_pcTileSelect->GetRect(), dstrect);
}

void CGameMain::DrawAttackableTile(SDL_Rect* destrect)
{
	DrawBackSurface(m_pcTileRed->GetSurface(), m_pcTileRed->GetRect(), destrect);
}

void CGameMain::DrawMoveableTile(SDL_Rect* destrect)
{
	int iTileX = destrect->x / TILE_SIZE;
	int iTileY = destrect->y / TILE_SIZE;
	if(GetCharacterManage()->GetCharacterParent(iTileX, iTileY))
		return;

	DrawBackSurface(m_pcTileWhite->GetSurface(), m_pcTileWhite->GetRect(), destrect);
}

bool CGameMain::isShowCursor()
{
	if(m_pcSelectCharacter && m_pcSelectCharacter->isMenu()){
		return FALSE;
	}
	return TRUE;
}

void CGameMain::Scroll()
{
	if(m_nMouseX < 30)
	{
		m_nScrollX -= SCROLL_VALUE;
	}
	else if(610 < m_nMouseX)
	{
		m_nScrollX += SCROLL_VALUE;
	}
	if(m_nMouseY < 30)
	{
		m_nScrollY -= SCROLL_VALUE;
	}
	else if(450 < m_nMouseY)
	{
		m_nScrollY += SCROLL_VALUE;
	}

	if(m_nScrollX < 0)
	{
		m_nScrollX = 0;
	}
	else if(MAX_SCROLL_X - GetWidth() < m_nScrollX)
	{
		m_nScrollX = MAX_SCROLL_X - GetWidth();
	}

	if(m_nScrollY < 0)
	{
		m_nScrollY = 0;
	}
	else if(MAX_SCROLL_Y - GetHeight() < m_nScrollY)
	{
		m_nScrollY = MAX_SCROLL_Y - GetHeight();
	}
}

void CGameMain::DrawBackSurface(SDL_Surface* surface, SDL_Rect* srcrect, SDL_Rect* dstrect)
{
	Blit(surface, srcrect, m_BackSurface, dstrect);
}

void CGameMain::KeyUp(int keyvalue)
{
	CSDLMain::KeyUp(keyvalue);

	if(keyvalue == SDLK_RETURN)
	{
		SetInputMode(false);
		SetFadeOut();
	}

	if(keyvalue == SDLK_SPACE)
	{
		SetFadeOut();
		SetInputMode(false); 
		m_pcGuiManage->SetPreemitiveGui(m_pcLogoGui);
	}

	if(keyvalue == SDLK_ESCAPE)
	{
		m_pcGuiManage->SetPreemitiveGui(NULL);
	}
}

void CGameMain::UnselectCharacter()
{
	if(m_pcSelectCharacter){
		m_pcSelectCharacter->SetCharacterState(CHARACTERSTATE_NONE);
		m_pcSelectCharacter = NULL;
	}
}

void CGameMain::Damage()
{
	if(m_pcEffectCharacter)
		m_pcEffectCharacter->SetDamage(true);

	static int i = 0;
	i += 5;
	if(50 <= i)
	{
		i = 0;
		EffectEnd();
	}
}

void CGameMain::EffectEnd()
{
	if(m_pcEffectCharacter)
	{
		m_pcEffectCharacter->SetDamage(false);
		m_pcEffectCharacter = NULL;
	}

	__super::EffectEnd();
}

void CGameMain::MouseMove(int x, int y)
{
	CSDLMain::MouseMove(x, y);

	Scroll();
}

void CGameMain::MouseButtonDown(MOUSE_BUTTON eMouseButton, int x, int y)
{
	if(eMouseButton == MOUSEBUTTON_LEFT)
	{
		int iTileX = (m_nScrollX + x) / TILE_SIZE;
		int iTileY = (m_nScrollY + y) / TILE_SIZE;
		if(m_pcSelectCharacter == NULL)
		{
			m_pcSelectCharacter = GetCharacterManage()->SelectCharacter(iTileX, iTileY);
			if(m_pcSelectCharacter)
			{
				m_pcSelectCharacter->SetCharacterState(CHARACTERSTATE_MENU);
				return;
			}
		}

		if(m_pcSelectCharacter)
		{
			CCharacterParent* pcCharacterParent = GetCharacterManage()->GetCharacterParent(iTileX, iTileY);
			if(m_pcSelectCharacter->isAttack() && GetCharacterManage()->isValidArea(m_pcSelectCharacter, pcCharacterParent, m_pcSelectCharacter->GetAttackableTile()))
			{
				m_pcEffectCharacter = pcCharacterParent;
				SetEffect((EffectFuncPtr)(&CGameMain::Damage));
				return;
			}

			if(GetCharacterManage()->MoveCharacterParent(m_pcSelectCharacter, iTileX, iTileY))
			{
				UnselectCharacter();
			}
			return;
		}
	}
	else if(eMouseButton == MOUSEBUTTON_RIGHT)
	{
		UnselectCharacter();
	}
}